//
// jMax
// Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// See file LICENSE for further informations on licensing terms.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// Based on Max/ISPW by Miller Puckette.
//
// Authors: Maurizio De Cecco, Francois Dechelle, Enzo Maggi, Norbert Schnell.
// 

package ircam.jmax.fts;

import ircam.jmax.*;

/**
 * A generic FTS object with an int value.
 * Used for intbox and sliders, for example.
 * If the listener of this object is an instance
 * of FtsIntValueListener, fire it when the we got a new value
 * from the server.
 */

public class FtsIntValueObject extends FtsObject
{
  /*****************************************************************************/
  /*                                                                           */
  /*                               CONSTRUCTORS                                */
  /*                                                                           */
  /*****************************************************************************/

  int value; 

  /* for the message box */
    public FtsIntValueObject(Fts fts, FtsObject parent, String variable, String className, int nArgs, FtsAtom args[])
    {
	super(fts, parent, null, className, className);
    }

    public FtsIntValueObject(Fts fts, FtsObject parent, String className, String description)
    {
	super(fts, parent, null, className, description);
    }

    /** Set the value. Tell it to the server, also */

    public void setValue(int value)
    {
	this.value = value;
	getFts().getServer().putObjectProperty(this, "value", value);
    }

    /** Get the current value */

    public int getValue()
    {
	return value;
    }

    /** Ask the server for the latest value */

    public void updateValue()
    {
	getFts().getServer().askObjectProperty(this, "value");
    }
       
    /* Over write the localPut message to handle value changes;
     */

    protected void localPut(String name, int newValue)
    {
	if (name == "value")
	    {
		value = newValue;
		
		if (listener instanceof FtsIntValueListener)
		    ((FtsIntValueListener) listener).valueChanged(newValue);
	    }
	else
	    super.localPut(name, newValue);
    }
}






